
% The abalone dataset from the UCI Machine Learning repository

raw_data = load('abalone.dat');
raw_data = raw_data(:, 1:7);
sample_size = size(raw_data, 1);

num_latents  = 2;
num_observed = 7;

n_types = ones(num_latents + num_observed, 1);
n_types(1:num_latents) = 0;

graph = zeros(num_latents + num_observed);

% Size
for i = 1:3
    graph(num_latents + i, 1) = 1;
end
% Weight
for i = 4:7
    graph(num_latents + i, 2) = 1;
end

%Structural model
graph(2, 1) = 1;

% Verify if cross-validation files exist
NUM_FOLDS = 5;
cv_file = fopen('abalone.cv', 'r');
if cv_file == -1 % Create file if it doesn't exist
   cv_file = fopen('abalone.cv', 'w');
   cv_matrix = get_cv_matrix(sample_size, NUM_FOLDS);
   for f = 1:NUM_FOLDS
       lcv = length(cv_matrix{f});
       for n = 1:lcv
           fprintf(cv_file, '%d ', cv_matrix{f}(n));             
       end
       fprintf(cv_file, '\n');
   end
   fclose(cv_file);
   cv_file = fopen('abalone.cv', 'r');
end

% Read test points from cross-validation file
all_test_points = cell(NUM_FOLDS, 1);
cv_matrix = get_cv_matrix(sample_size, NUM_FOLDS); % Just to get sizes
for f = 1:NUM_FOLDS
   lcv = length(cv_matrix{f});
   data_line = textscan(cv_file, '%d', lcv);
   all_test_points{f} = data_line{1}';
end
fclose(cv_file);

all_train_points = cell(NUM_FOLDS, 1);
for f = 1:NUM_FOLDS
    all_train_points{f} = setdiff(1:sample_size, all_test_points{f});
end

test_vars = [];    % Which variables are going to be predicted
anchors   = [1 4]; % Variables of single coefficient, which are set to
                   % have unit slop loading and zero intercept.

latent_data = [];
LOG_NAME    = 'mse-gpr-abalone.log';
num_x_mix   = 3; 

% Generate missing at random matrix
if exist('abalone.mars.mat', 'file')
   load('abalone.mars.mat', 'mars_matrix');
else
   mars_matrix = rand(size(raw_data)) > 0.5;
   mars_matrix(:, test_vars) = 0;
   save 'abalone.mars.mat' 'mars_matrix'
end

