    
% Generate a synthetic dataset with traint/test split

raw_data = load('consumer.dat');
raw_data = raw_data(:, setdiff(1:19, [6 7 19])); % No variables 6, 7, 19

sample_size = size(raw_data, 1);

num_latents  = 4;
num_observed = 16;

n_types = ones(num_latents + num_observed, 1);
n_types(1:num_latents) = 0;

graph = zeros(num_latents + num_observed);

% Structural model
graph(2, 1) = 1; graph(2, 3) = 1;
graph(3, 1) = 1; 
graph(4, 2) = 1;

% Measurement model
graph(num_latents +  1, 1) = 1;
graph(num_latents +  2, 1) = 1;
graph(num_latents +  3, 1) = 1;
graph(num_latents +  4, 1) = 1;
graph(num_latents +  5, 1) = 1;

graph(num_latents +  6, 2) = 1;
graph(num_latents +  7, 2) = 1;
graph(num_latents +  8, 2) = 1;
graph(num_latents +  9, 2) = 1;
graph(num_latents + 10, 2) = 1;

graph(num_latents + 11, 3) = 1;
graph(num_latents + 12, 3) = 1;
graph(num_latents + 13, 3) = 1;
graph(num_latents + 14, 3) = 1;

graph(num_latents + 15, 4) = 1;
graph(num_latents + 16, 4) = 1;
graph(num_latents +  9, 4) = 1;

% Verify if cross-validation files exist
NUM_FOLDS = 5;
cv_file = fopen('consumer.cv', 'r');
if cv_file == -1 % Create file if it doesn't exist
   cv_file = fopen('consumer.cv', 'w');
   cv_matrix = get_cv_matrix(sample_size, NUM_FOLDS);
   for f = 1:NUM_FOLDS
       lcv = length(cv_matrix{f});
       for n = 1:lcv
           fprintf(cv_file, '%d ', cv_matrix{f}(n));             
       end
       fprintf(cv_file, '\n');
   end
   fclose(cv_file);
   cv_file = fopen('consumer.cv', 'r');
end

% Read test points from cross-validation file
all_test_points = cell(NUM_FOLDS, 1);
cv_matrix = get_cv_matrix(sample_size, NUM_FOLDS); % Just to get sizes
for f = 1:NUM_FOLDS
   lcv = length(cv_matrix{f});
   data_line = textscan(cv_file, '%d', lcv);
   all_test_points{f} = data_line{1}';
end
fclose(cv_file);

all_train_points = cell(NUM_FOLDS, 1);
for f = 1:NUM_FOLDS
    all_train_points{f} = setdiff(1:sample_size, all_test_points{f});
end

%test_points = test_points{1}; train_points = train_points{1};

test_vars = [];    % Which variables are going to be predicted
anchors = [5 10 12 15]; % Variables of single coefficient, which are set to
                        % have unit slop loading and zero intercept.
latent_data = [];
LOG_NAME    = 'mse-gpr-consumer.log';
num_x_mix   = 5; 

% Generate missing at random matrix
if exist('consumer.mars.mat', 'file')
   load('consumer.mars.mat', 'mars_matrix');
else
   mars_matrix = rand(size(raw_data)) > 0.5;
   mars_matrix(:, test_vars) = 0;
   save 'consumer.mars.mat' 'mars_matrix'
end

