% Simple script that assumes some dataset is loaded with an indication of 
% training and test points. It then prepares logs for cross-validation.

% First, run a script that will generated the required data structures. We
% provide a few demos: load_synth, load_depress, load_consumer, load_house
% and load_abalone.

% In all of these demos, 5-fold cross validation is setup. Change
% NUM_FOLDS to a smaller number if you want the experiment to finish
% earlier.

num_mcmc_samples = 12000;
burn_in          = 2000;

M = 50; % Size of pseudopoint set

BATCH_SIZE           = 5000; % Number of points kept in main memory
use_starting         = 0;    % If 1, initialize model with a parametric one
initial_mcmc_samples = 5000; % Number of samples for the initialization procedure

% Comment out next line in order to use the data with missing entries
mars_matrix = zeros(size(raw_data));

for f = 1:NUM_FOLDS
    
    train_points = all_train_points{f}; %#ok<USENS>
    test_points  = all_test_points{f};  %#ok<USENS>
    data = normalize_training(raw_data, train_points, mars_matrix);    
    if isempty(test_vars)
       input_data = data(train_points, :);
       mars_matrix = zeros(size(input_data));
       test_values = [];
    else
       input_data = data; if sum(sum(mars_matrix))> 0, input_data(mars_matrix) = 0; end
       test_values = zeros(length(test_points) * length(test_vars), 2);
       tp = 1;
       for t = test_points    
           for v = test_vars    
               test_values(tp, 1) = t;
               test_values(tp, 2) = v;
               tp = tp + 1;
           end
       end
    end        

    [test_pred_gpp, sampled_x_gpp, sampled_f_gpp, sampled_lin_gpp, ...
     sampled_xerr_var_gpp, sampled_yerr_var_gpp, sampled_kernel_hyper_gpp, pseudo_rho] = ...
       run_gpsem_mcmc_spgp(num_mcmc_samples, M, graph, input_data, n_types, ...
                           num_x_mix, anchors, test_values, latent_data, ...
                           BATCH_SIZE, use_starting, initial_mcmc_samples);

    copyfile('sample_x.txt', strcat('sample_x.', int2str(f), '.log'));
    copyfile('sample_y.txt', strcat('sample_y.', int2str(f), '.log'));
    copyfile('sample_lambda.txt', strcat('sample_lambda.', int2str(f), '.log'));
    copyfile('sample_hgp.txt', strcat('sample_hgp.', int2str(f), '.log'));
    copyfile('sample_z.txt', strcat('sample_z.', int2str(f), '.log'));
    copyfile('sample_yvar.txt', strcat('sample_yvar.', int2str(f), '.log'));
    copyfile('sample_xvar.txt', strcat('sample_xvar.', int2str(f), '.log'));
    copyfile('sample_f.txt', strcat('sample_f.', int2str(f), '.log'));
    copyfile('sample_pseudof.txt', strcat('sample_pseudof.', int2str(f), '.log'));
    copyfile('sample_pseudox.txt', strcat('sample_pseudox.', int2str(f), '.log'));
    copyfile('sample_exo.txt', strcat('sample_exo.', int2str(f), '.log'));
    
    clear sampled_x; clear sampled_f;
end

